/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.iptc.vocabularies;

import java.util.List;
import it.tidalwave.metadata.iptc.vocabularies.IPTCVocabulary.Entry;
import it.tidalwave.metadata.iptc.vocabularies.impl.IPTCVocabularyFactoryImpl;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class IPTCVocabularyFactoryTest 
  {
    private IPTCVocabularyFactory factory;
    
    @Before
    public void setUp() 
      {
        factory = new IPTCVocabularyFactoryImpl();
      }

    @After
    public void tearDown()   
      {
        factory = null;
      }
    
    @Test
    public void testLocator()
      {
        final IPTCVocabularyFactory vocabularyFactory = IPTCVocabularyFactory.Locator.findIPTCVocabularyFactory(); 
        assertNotNull(vocabularyFactory);
        assertTrue(vocabularyFactory instanceof IPTCVocabularyFactoryImpl);
      }

    @Test
    public void testIPTCGenreVocabularyIsSingleton()
      {
        final Object o1 = factory.getGenreVocabulary();
        final Object o2 = factory.getGenreVocabulary();
        assertNotNull(o1);
        assertNotNull(o2);
        assertTrue(o1 == o2);
      }
    
    @Test
    public void testIPTCGenreVocabulary() 
      throws Exception
      {
        assertVocabulary(factory.getGenreVocabulary(), 44, "Actuality", "Wrapup");
      }
    
    @Test
    public void testIPTCProviderVocabularyIsSingleton()
      {
        final Object o1 = factory.getProviderVocabulary();
        final Object o2 = factory.getProviderVocabulary();
        assertNotNull(o1);
        assertNotNull(o2);
        assertTrue(o1 == o2);
      }
    
    @Test
    public void testIPTCProviderVocabulary() 
      throws Exception
      {
        assertVocabulary(factory.getProviderVocabulary(), 68, "Agence France-Presse (Company)", "Yemen News Agency (Company)");
      }
    
    @Test
    public void testIPTCSceneVocabularyIsSingleton()
      {
        final Object o1 = factory.getSceneVocabulary();
        final Object o2 = factory.getSceneVocabulary();
        assertNotNull(o1);
        assertNotNull(o2);
        assertTrue(o1 == o2);
      }
    
    @Test
    public void testIPTCSceneVocabulary() 
      throws Exception
      {
        assertVocabulary(factory.getSceneVocabulary(), 24, "action", "under-water");
      }
    
    @Test
    public void testIPTCSubjectCodeVocabularyIsSingleton()
      {
        final Object o1 = factory.getSubjectCodeVocabulary();
        final Object o2 = factory.getSubjectCodeVocabulary();
        assertNotNull(o1);
        assertNotNull(o2);
        assertTrue(o1 == o2);
      }
    
    @Test
    public void testIPTCSubjectCodeVocabulary() 
      throws Exception
      {
        assertVocabulary(factory.getSubjectCodeVocabulary(), 1402, "(US) National Football League (NFL) (North American)", "zoology");
      }
    
    @Test
    public void testIPTCCopyrightStatusVocabularyIsSingleton()
      {
        final Object o1 = factory.getCopyrightStatusVocabulary();
        final Object o2 = factory.getCopyrightStatusVocabulary();
        assertNotNull(o1);
        assertNotNull(o2);
        assertTrue(o1 == o2);
      }
    
//    @Test
//    public void testIPTCCopyrightStatusCodeVocabulary() 
//      throws Exception
//      {
//        TODO assert sample values
//      }
    
    private void assertVocabulary (final IPTCVocabulary vocabulary, final int size, final String firstNane, final String lastName)
      {
        assertNotNull(vocabulary);
        final List<Entry> entries = vocabulary.getFullEntries();
        assertNotNull(entries);
        assertEquals(size, entries.size());
        assertEquals(firstNane, entries.get(0).getName());
        assertEquals(lastName, entries.get(entries.size() - 1).getName());
      }
  }