/***********************************************************************************************************************
 *
 * blueMarine Metadata - open source media workflow
 * Copyright (C) 2007-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluemarine.tidalwave.it
 * SCM: https://kenai.com/hg/bluemarine~metadata-src
 *
 **********************************************************************************************************************/
package it.tidalwave.metadata.iptc.vocabularies.impl;

import java.util.Iterator;
import java.util.Collection;
import java.util.concurrent.CopyOnWriteArrayList;
import it.tidalwave.metadata.iptc.vocabularies.IPTCVocabulary.Entry;
import it.tidalwave.metadata.iptc.vocabularies.impl.IPTCVocabularyImpl.EntryImpl;
import org.junit.Test;
import static org.junit.Assert.*;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class IPTCVocabularyImplTest 
  {
    @Test 
    public void testGetEntries()
      {
        final IPTCVocabularyImpl vocabulary = new IPTCVocabularyImpl();
        
        for (int i = 9; i >= 0; i--)
          {
            vocabulary.add(new EntryImpl(null, "name" + i, null, null, null, null, null));  
          }
        
        vocabulary.prepare();
        final Collection<String> entries = vocabulary.getEntries();
        assertNotNull(entries);
        assertNull(entries.iterator().next());
        assertTrue(entries instanceof CopyOnWriteArrayList);
        ((CopyOnWriteArrayList)entries).remove(0);
        assertStringsAreSorted(entries);
      }

    @Test 
    public void testEntriesAreOrdered()
      {
        final IPTCVocabularyImpl vocabulary = new IPTCVocabularyImpl();
        
        for (int i = 9; i >= 0; i--)
          {
            vocabulary.add(new EntryImpl(null, "name" + i, null, null, null, null, null));  
          }
        
        vocabulary.prepare();
        assertEntriesAreSorted(vocabulary.internalGetEntries());
      }

    @Test
    public void testEntryImpl() 
      {
        final String formalName = "formalname";
        final String name = "name";
        final String explanation = "explanation";
        final String changeComment = "changeComment";
        final String changeVersion = "changeVersion";
        final String firstVersion = "firstVersion";
        final String deprecatedVersion = "deprecatedVersion";
        IPTCVocabularyImpl.EntryImpl entry = new IPTCVocabularyImpl.EntryImpl(formalName, name, explanation, changeComment, changeVersion, firstVersion, deprecatedVersion);
        assertEquals(formalName, entry.getFormalName());
        assertEquals(name, entry.getName());
        assertEquals(explanation, entry.getExplanation());
        assertEquals(changeComment, entry.getChangeComment());
        assertEquals(changeVersion, entry.getChangeVersion());
        assertEquals(firstVersion, entry.getFirstVersion());
        assertEquals(deprecatedVersion, entry.getDeprecatedVersion());
      }
    
    @Test
    public void testToString()
      {
        final Entry entry = new EntryImpl(null, "name", null, null, null, null, null);  
        assertEquals("Entry[name]", entry.toString());
      }

    private void assertEntriesAreSorted (final Collection<Entry> entries)
      {
        final Iterator<Entry> i = entries.iterator();
        Entry e1 = i.next();

        while (i.hasNext())  
          {
            final Entry e2 = i.next();
            assertTrue(e1.getName().compareTo(e2.getName()) < 0);
            e1 = e2;
          }
      }
    
    private void assertStringsAreSorted (final Collection<String> strings)
      {
        final Iterator<String> i = strings.iterator();
        String e1 = i.next();

        while (i.hasNext())  
          {
            final String e2 = i.next();
            assertTrue(e1.compareTo(e2) < 0);
            e1 = e2;
          }
      }
  }